# -*- coding: utf-8 -*-
"""
Created on Wed Jun  9 17:39:13 2021

@author: Zineb Bouaoudate
"""
import numpy as np
import matplotlib.pyplot as plt
import h5py as hp
import os, sys, glob

#=============================================================================
from pythermica import Thermica
from pathlib import Path


plot_style = {
    "axes.grid": True,
    "axes.grid.axis": 'y',
    "axes.labelsize": 12,
    "axes.titlesize": 16,
    "figure.autolayout": True,
    "figure.dpi": 300,
    "figure.figsize": [6.4, 4.8],
    "font.family": "STIXGeneral",
    "grid.alpha": 0.5,
    "grid.color": "grey",
    "grid.linestyle": "dotted",
    "grid.linewidth": 0.8,
    "image.aspect": "auto",
    "image.cmap": "plasma",
    "image.interpolation": "nearest",
    "image.origin": "lower",
    "lines.linewidth": 1,
    "lines.markersize": 5,
    "mathtext.fontset": "stix",
    "savefig.format": "eps",
    "xtick.labelsize": 10,
    "xtick.minor.visible": True,
    "xtick.top": True,
    "ytick.labelsize": 10,
    "ytick.minor.visible": True,
    "ytick.right": True
}

plt.rcParams.update(plot_style)

def figure_over_nodes(therm_results,
                      temperatures,
                      times,
                      n_orbits,
                      path_list, case_names,
                      nodes_to_process = ["/Powercard P60", "/Printed Circuits",
                     "/Transponders (1)", "/Batteries"] ,
                      path_root = "./",
                      name_yaxis="Temperature",
                      filename_prefix=""):
    
    """
    generate a bunch of figures to compare simulations
    """
    
    n_path = len(path_list)
    period = times[0][-1] / n_orbits
    
    nodes_processed = []

    for idx_node, node_label in enumerate(therm_results[0].names_unique):
        """looping over all the object names, hopping they are the same between the different cases"""
        
        if node_label in nodes_to_process:
            
            nodes_processed.append(nodes_to_process)
            
            temperatures_node_average = []  # n_path lists of node-averaged temperature corresponding to n_path cases
            temperature_max = []  # maximum temperatures corresponding to n_path cases
            temperature_min = []  # minimum temperatures corresponding to n_path cases
            temperature_time_average = []  # time-averaged temperatures corresponding to n_path cases
            
            plt.figure(figsize=(10,4))
            
            for idx_path in range(n_path):
                
                index_nodes = []
                
                for i, node in enumerate(therm_results[idx_path].nodes_value):
                    if node in  therm_results[idx_path].nodes_per_names[idx_node]:
                        index_nodes.append(i)
                        
                index_nodes = np.array(index_nodes)  # index of nodes corresponding to label
                # print("the nodes for "+node_label+" are :", nodes_list_for_label)
                # print("the index nodes are :", index_nodes)
                    
                _max_temp = []
                _min_temp = []
                _average_temp = []
                for _temp in temperatures[idx_path].T[:, index_nodes]:
                    _max_temp.append(max(_temp))
                    _min_temp.append(min(_temp))
                    _average_temp.append(np.mean(_temp))
                temperatures_node_average.append(_average_temp)
                temperature_max.append(max(_max_temp))
                temperature_min.append(min(_min_temp))
                temperature_time_average.append(np.mean(_average_temp))
                
            
            
                line, = plt.plot(times[idx_path], temperatures_node_average[idx_path],
                         label=case_names[idx_path] + f" (max= {temperature_max[idx_path]:2.2f}, min=" +
                               f"{temperature_min[idx_path]:2.2f}, average=" +
                               f"{temperature_time_average[idx_path]:2.2f} over {len(index_nodes)} nodes)")
                
                for _temp in temperatures[idx_path].T[:, index_nodes].T:
                    plt.plot(times[idx_path], _temp, color=line.get_color(), alpha=0.3)
                    
                plt.hlines(temperature_time_average[idx_path],
                           times[idx_path][0],
                           times[idx_path][-1],
                           colors=line.get_color())
                
            thermobject = therm_results[0]
            
            solarFlux, time_sf = thermobject.read_solarflux("Results", "Direct Solar", "Flux")
            solarFlux = solarFlux.mean(axis=0)
            solarFlux[solarFlux>0] =  max(temperature_max)
            solarFlux[solarFlux<=0] = min(temperature_min)
            
            plt.fill_between(time_sf,
                             solarFlux,
                             min(temperature_min),
                             color="y", alpha=0.5, zorder=-999,
                             linewidth=0)
            
            plt.vlines([period*k for k in range(1, n_orbits+1)], max(temperature_max), min(temperature_min), linestyles="--")
            plt.title(node_label[1:])
            plt.xlabel('Time (hr)')
            plt.ylabel(name_yaxis)
            plt.legend(loc="upper center", bbox_to_anchor=(0.5, -0.15))
            
            plt.tight_layout()
            
            figure_name = filename_prefix + node_label[1:] + '.png'  # droping the first char which is "/"
            
            p_root = Path(path_root)
            p_img_root = p_root / "images"
            if not p_img_root.exists():
                p_img_root.mkdir()
                
            plt.savefig(p_img_root / figure_name)
            plt.close()

        
        for node in nodes_to_process:
            if node not in nodes_processed:
                print(f"WARNING ! the node {node} has not been processed")


def totalInternalDissipation(therm_results,
                      internal_dissipations,
                      time,
                      n_orbits,
                      path_list, case_names,
                      nodes_to_process = None ,
                      path_root = "./",
                      filename_prefix="Total_IQ"):
    
    """
    generate a bunch of figures to compare simulations
    """
    
    n_path = len(path_list)
    period = time[0][-1] / n_orbits
    
    
    plt.figure(figsize=(10,4))
    
    
    for idx_path in range(n_path):
        
        total_IQ = np.zeros_like(time[idx_path])
        
        if nodes_to_process is None :
            list_to_process = therm_results[idx_path].names_unique
        else :
            list_to_process = nodes_to_process
        

        for idx_node, node_label in enumerate(therm_results[idx_path].names_unique):
            """looping over all the object names, hopping they are the same between the different cases"""
            
            if node_label in list_to_process:

                index_nodes = []
                
                for i, node in enumerate(therm_results[idx_path].nodes_value):
                    if node in  therm_results[idx_path].nodes_per_names[idx_node]:
                        index_nodes.append(i)
                        
                index_nodes = np.array(index_nodes)  # index of nodes corresponding to label
                # print("the nodes for "+node_label+" are :", nodes_list_for_label)
                # print("the index nodes are :", index_nodes)
                    

                for _IQ in internal_dissipations[idx_path][index_nodes, :]:
                    
                    total_IQ += _IQ
                    

           
            
        line, = plt.plot(time[idx_path], total_IQ,
                 label=case_names[idx_path],
                 )
        
        
    ax = plt.gca()
    
    for k in range(1, n_orbits+1):
        ax.axvline(period*k, ls="--")
    
    titletext = "Total internal dissipation power"
    
    if nodes_to_process is not None:
        titletext += "\n"
        for nodename in nodes_to_process:
            titletext += nodename + ", "
                
    plt.title(titletext)    
    plt.xlabel('Time (hr)')
    plt.ylabel("internal dissipations [W]")
    plt.legend(loc="upper center", bbox_to_anchor=(0.5, -0.15))
    
    p_root = Path(path_root)
    p_img_root = p_root / "images"
    if not p_img_root.exists():
        p_img_root.mkdir()

    figure_name = filename_prefix + '.png'

    plt.savefig(p_img_root / figure_name)
    plt.close()
    



# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2021-12-09 19:11:50
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-26 14:31:06
# the main script, mainly to test and develop the package

from fileinput import filename
import numpy as np
import matplotlib.pyplot as plt
import h5py as hp
import os, sys, glob
from pathlib import Path

#=============================================================================

class Thermica():
    """Main class to analyse the simulation results"""

    def __init__(self, path, verbose=True):
        """Init the class"""
        
        if type(path) is Path:
            self.path = path  #: path were the simulation results are stored
        elif type(path) is str:
            self.path = Path(path)  #: path were the simulation results are stored
        else:
            raise RuntimeError("path Not with rigth Type")
        self.names_unique = []  #: list of the names used to label the nodes
        self.nodes_per_names = [[]]  #: list of the lists of the nodes for each unique names
        self.nodes_value = np.array([])  #: list of the node value, order in the same way as the temperatures
        self.verbose = verbose
        
        if self.verbose:
            print("Initializing Thermica at path :")
            print(path)
            print("Files present in the path :")
            [ print(n) for n in self.path.glob("*.h5" )]

        try:
            self.process_model_nodes()
        except:
            raise RuntimeWarning("Cannot process the nodes. Maybe the temperature file is not present.")
        




    def get_filenames(self, type="none", sort_by="name"):
        """general fonction to get the filenames depending of the type of results wanted"""
        
        if type == "solarflux":
            extention = "*.sf.h5"
        elif type == "earthflux":
            extention = "*.pf.h5"
        elif type == "temperature":
            extention = "*.temp.h5"

        files =  list(self.path.glob(extention))
        if sort_by == "date":
            files.sort(key=os.path.getmtime)
        else:
            files.sort()

        return files
 
 
    def get_solarflux_file(self):
        """return the h5 file name with the solar flux"""
 
        filenames = self.get_filenames("solarflux")
        
        if self.verbose:
            print(filenames)
            
        if len(filenames) == 0:
            raise RuntimeError("Error, missing Solaf Flux file")
        elif len(filenames) > 1:
            print("WARNING : Multiple files match the Solar Flux extention")

 
        filename = filenames[0]
 
        return filename
 
 
    def get_temperature_file(self):
        """return the h5 file name with the temperature data"""
 
        filenames = self.get_filenames("temperature")
        
        if len(filenames) == 0:
            raise RuntimeError("No temperature file found")
        elif len(filenames) > 1:
            print("WARNING : Multiple files match the temperature extention")
        
        if self.verbose:
            print("\n the temperature files are :")
            print(filenames)
            
        filename = filenames[0]
        return filename
    
    
    def read_solarflux(self, groupname ="Results",
                       subgroupname = "Direct Solar",
                       datasetname  = "Flux"):
        
        solarfluxname = self.get_solarflux_file()
 
        with hp.File(solarfluxname, "r") as h5file:
            value = h5file[groupname][subgroupname][datasetname][()]
            time = h5file["Time"]["Computed times"][()]
 
        time -= time[0]
        time *= 24
        return value, time
 
 
    def read_temperature_results(self, groupname="Results", subgroup="Thermal", datasetname="Temperatures"):
        """Read the *.temp.h5 data file according to the group, subgroup and dataset name

        Args:
            groupname (str): name of the groupe, from
                            ["Model", "Posther"n "Results", "Run Info", "Time"].
                            Defautl is "Results"
            subgroup (str): name of the subgroup. If Group is "Results", options are
                            ["Couplings", "Electric", "Outgassing", "Properties", "Thermal", "Variables"].
                            Default is Thermal
            datasetname (str): name of the DataSet. If Subgroup is "Thermal", options are
                            ["Albedo fluxes", "Internal dissipations", "Planet IR fluxes", "Residual fluxes", "Solar fluxes", "Temperatures"].
                            Default is "Temperatures"

        Returns:
            [type]: [description]
        """
        filename = self.get_temperature_file()
 
        with hp.File(filename, "r") as h5file:
            value = h5file[groupname][subgroup][datasetname][()]
 
        return value


    def get_temperature(self):

        return self.read_temperature_results(
                                          groupname="Results",
                                          subgroup="Thermal",
                                          datasetname="Temperatures" )


    def get_internal_dissipation(self) :
        """return the value in W of the internal dissiabation for each node (each mesh cell)"""


        return self.read_temperature_results(groupname="Results", subgroup="Thermal",
                                         datasetname="Internal dissipations")

 
    def read_temperature_results2(self, groupname, datasetname):
        """Open the temperature h5 file, but access the dataset with only one groupe of hyerachi"""
        filename = self.get_temperature_file()
 
        with hp.File(filename, "r") as h5file:
            
            value = h5file[groupname][datasetname][()]
 
        return value
    
    
    def return_time_temperature(self):
        """the time vector is stor in all of the h5 files"""
 
        filename = self.get_temperature_file()
 
        with hp.File(filename, "r") as h5file:
            value = h5file["Time"]["Frequency 1"][()]
 
        value -= value[0]
        value *= 24
        return value


    def process_model_nodes(self):
        """The nodes are accessible, but this model is meat to understand better their information
        """
        
        try :
            node_liste = self.read_temperature_results2("Model", "Nodes")
        except OSError :
            raise RuntimeError("Cannot open the file. You should close any application using the file")

        self.node_liste = np.array([n[1] for n in node_liste])

        names = [d[2] for d in node_liste]
        self.nodes_value = np.array([d[1] for d in node_liste])
 
        names_unique = []
        for n in names:
            name_in_str = n.decode("utf-8")
            if not name_in_str in names_unique:
                if len(name_in_str) > 0:
                    names_unique.append(n.decode("utf-8"))
        self.names_unique = names_unique
 
        nodes_per_names = [[] for n in self.names_unique]
 
        for i, n in enumerate(self.names_unique):
            for b in node_liste:
                if b[2].decode("utf-8") == n:
                    nodes_per_names[i].append(b[1])
        self.nodes_per_names = nodes_per_names
 
        if self.verbose :
            print("We have some names :")
            print(self.names_unique)
            print()
            for i, n in enumerate(self.names_unique):
                print("nodes in ", n)
                print(self.nodes_per_names[i])
 
    
    def dump_result_file_structure(self):
        """Print the structure of the file
        """
        from pythermica.hdf5_process import h5dump
        
        filename = self.get_temperature_file()
        h5dump(filename)


# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-24 16:07:45
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-24 16:59:59
import pytest
import shutil
import os, sys
import pythermica
from pythermica import Thermica
from pythermica.plot_variables import (plot_style,
                                       figure_over_nodes,
                                       totalInternalDissipation,
                                        )
                                        
from pathlib import Path

def test_plot_style():
    
    assert type(plot_style) == dict
    
def test_figure_over_nodes():
    
    path = pythermica.__path__[0]+"/../exemples/simulation_1/results_1/"

    therm_results = Thermica(path, verbose=False)

    time_temperature = therm_results.return_time_temperature()
    temperatures = therm_results.get_temperature()
    
    figure_over_nodes(therm_results = [therm_results],
                      temperatures=[temperatures],
                      times=[time_temperature],
                      n_orbits=8,
                      path_list=[path],
                      case_names=["Default"],
                      nodes_to_process=["/Powercard P60", "/Printed Circuits",
                                        "/Transponder (1)", "/Batteries"],
                      path_root="./",
                      name_yaxis="Temperature",
                      filename_prefix="")
    
    assert os.path.exists("./images")
    
    list_figures = list(Path("./images").glob("*.png"))
    
    assert len(list_figures) == 4
    
    shutil.rmtree('./images')

    
    
def test_plot_Internal_dissipation():
    path = pythermica.__path__[0]+"/../exemples/simulation_1/results_1/"

    therm_results = Thermica(path, verbose=False)
    time_temperature = therm_results.return_time_temperature()
    iq = therm_results.get_internal_dissipation()

    totalInternalDissipation( [therm_results],
                             [iq],
                             [time_temperature],
                             8,
                             [path],
                             ["default"],
                             )
    
    assert os.path.exists("./images")

    list_figures = list(Path("./images").glob("Total_IQ.png"))

    assert len(list_figures) == 1

    shutil.rmtree('./images')



# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2021-12-17 15:36:12
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-26 15:17:59

import numpy as np
import parse as prs
import matplotlib.pyplot as plt
from pathlib import Path

def get_useful_files(root):
    """Look for the useful fils in a folder

    Args:
        root (str): relative or absolute name of the folder
    """

    p_root = Path(root)

    filename_gb = list(p_root.glob("*gb.txt"))[0]
    filename_gl = list(p_root.glob("*gl.nwk"))[0]
    filename_gr = list(p_root.glob("*gr.nwk"))[0]
    filename_nod = list(p_root.glob("*.nod.nwk"))[0]
    
    return filename_gb, filename_nod, filename_gr, filename_gl

def extract_nodes(filename_nod):
    """Read the *.od.nwk file and generate the list of all nodes and their names

    Args:
        filename_nod (str): file name of the results in *.nod.nwk

    Returns:
        list_of_nodes_numbers [list]: list of all the node numbers, ordered

        list_of_nodes_names [list]: list of the name of each node,
                                    with the same order as list_of_nodes_numbers

    """

    useful_lines = []

    with open(filename_nod, 'r') as f_nod:

        lines = f_nod.readlines()
        
        for i, line in enumerate(lines):

            if line == "$NODES\n":
                for j, line2 in enumerate(lines[i:]):

                    if line2 != "$INITIAL\n":
                        useful_lines.append(line2) 
                    else:
                        break

    useful_lines = useful_lines[3:-4]

    list_of_nodes_numbers = []
    list_of_nodes_names = []

    detect_str = " = '/"

    for line in useful_lines:

        try :
            index = line.index(detect_str)

            list_of_nodes_numbers.append( eval( line[index-6:index]))
            list_of_nodes_names.append(  line[index+5:index+40].strip()  )

        except ValueError:
            pass

    return list_of_nodes_numbers, list_of_nodes_names

def analyse_GR_nwk_data(filename_gr: str,
                        list_of_nodes_numbers: list,
                        list_of_nodes_names: list ):
    """read the gr.nwk file and generate the relation matrix,
     with the same order as the list_of_nodes_numbers

    Args:
        filename_gr (str): filename of the *gr.nwk to process
        list_of_nodes_numbers (list): list of all the node numbers
        list_of_nodes_names (list): ist of the name of each node,
                                    with the same order as list_of_nodes_numbers

    Returns:
        mat_GR (np.array): the coefficient matrice with GL links between each nodes
    """
    

    mat_GR = np.zeros( (len(list_of_nodes_names), len(list_of_nodes_names)))

    with open(filename_gr, 'r') as f_gr:
        lines = f_gr.readlines()
        
    for line in lines:

        if "GR( " in line:

            
            format_string = "      GR( {n_org} , {n_dest} )	=	{data};\n"
            parsed = prs.parse(format_string, line)
            
            if parsed is None:
                format_string = "     #GR( {n_org} , {n_dest} )	=	{data};\n"
                parsed = prs.parse(format_string, line)

            if parsed is None:
                format_string = "     #GR( {n_org} , {n_dest} )	=	{data};	# {otherdata} %\n"
                parsed = prs.parse(format_string, line)

            
            try:
                gr_value = eval( parsed["data"])
                
                if parsed["n_dest"]  not in ['99999999', "SCREEN", "FILTER"]:
                    ind_dest = list_of_nodes_numbers.index(eval(parsed["n_dest"]))
                    ind_org = list_of_nodes_numbers.index(eval(parsed["n_org"]))

                    mat_GR[ind_dest, ind_org] = gr_value
                    mat_GR[ind_org, ind_dest] = gr_value
            except TypeError:
                print(line)

    return mat_GR

def analyse_GL_nwk_data(filename_gl, list_of_nodes_numbers, list_of_nodes_names):
    """read the gl.nwk file and generate the relation matrix,
     with the same order as the list_of_nodes_numbers

    Args:
        filename_gl (str): filename of the *gl.nwk to process
        list_of_nodes_numbers (list): list of all the node numbers
        list_of_nodes_names (list): ist of the name of each node,
                                    with the same order as list_of_nodes_numbers

    Returns:
        mat_GL (np.array): the coefficient matrice with GL links between each nodes
    """

    mat_GL = np.zeros( (len(list_of_nodes_names), len(list_of_nodes_names)))

    with open(filename_gl, 'r') as f_gr:
        lines = f_gr.readlines()
        
    COND_Aluminium_2024 = 120
    COND_Aluminium_6061 = 120
    COND_Aluminium_7075 = 120
    COND_MLI = 0


    for line in lines:

        if "GL(" in line:

            
            format_string = "   GL({n_org}, {n_dest}) =   {data};\n"
            parsed = prs.parse(format_string, line)

            if parsed is None:
                format_string = "   GL({n_org}, {n_dest}) = {data};\n"
                parsed = prs.parse(format_string, line)


            gl_value = eval( parsed["data"])

            ind_dest = list_of_nodes_numbers.index(eval(parsed["n_dest"]))
            ind_org = list_of_nodes_numbers.index(eval(parsed["n_org"]))

            mat_GL[ind_dest, ind_org] = gl_value
            mat_GL[ind_org, ind_dest] = gl_value
    
    return mat_GL

def generate_correlation_matrixes(list_mats,
                                  list_node_names,
                                  list_titles,
                                  use_log=True,
                                  save_fig=False,
                                  filename="",
                                  cmap="afmhot_r"):
    """creat a Matrice figure of the node coefficients

    Args:
        list_mats (list of np.array): list of the coefficient matrices
        list_node_names (list): list of the node names, needs one for each node
        list_titles (list): list of the subplot titles, coerrespondes to list_mats
        use_log (bool, optional): if `True`, then compute the log10 of the matrice before ploting . Defaults to True.
        save_fig (bool, optional): if `True`, then save the figure on a PNG file. Defaults to False.
        filename (str, optional): name of the figure file, used if `save_fig`. Defaults to "".

    Returns:
        fig, axarr [matplotlib.Figure and Axies]: the matplotlig objects
    """


    n_plots = len(list_mats)

    fig, axarr = plt.subplots(n_plots, 1, figsize=(15, 15*n_plots)) 

    if n_plots == 1:
        axarr = [axarr]


    for i, mat_values in enumerate(list_mats):

        """ preprocessing the matrices data"""
        #mat_GL[mat_GL < 1e-10] = 0
        mat_values /= mat_values.max()

        if use_log:
            mat_values_toplot = np.log10(mat_values+1e-5)
        else:
            mat_values_toplot = mat_values*10
        
        axarr[i].grid(False)
        neg = axarr[i].imshow( mat_values_toplot, cmap=cmap )
        fig.colorbar(neg, ax=axarr[i],
                    #  location='right',
                     anchor=(0.5, 1.5),
                     shrink=0.3,
                     label="Coef Value", )

        text_title = list_titles[i]
        if use_log:
            text_title += " \nLog Scale"

        axarr[i].set_title(text_title)


    for ax in axarr:
        """Set up the figures nicely"""
        #ax.set_xlabel("node Numbers")
        #ax.set_ylabel("node Numbers")
        ax.set_xticks(range(len(list_node_names)))
        ax.set_yticks(range(len(list_node_names)))

        ax.set_xticklabels(list_node_names)
        ax.tick_params(labeltop=True, labelright=True)

        for label in ax.get_xticklabels():
            pos = label.get_position()
            if pos[1] == 0:
                label.set_ha("right")
                label.set_rotation(80)
            elif pos[1] == 1:
                label.set_ha("left")
                label.set_rotation(80)

        ax.set_yticklabels(list_node_names)

        ax.grid(alpha=0.2)


    fig.tight_layout()

    if save_fig:
        if use_log:
            filename += "_log"

        plt.savefig(filename+".png", dpi=200)
    
    return fig, axarr

def parse_one_node_lines(lines_selected):
    node_numbers = []
    gb_vfs = []
    gb_irs = []

    for line in lines_selected:

        node_number = eval(line[26:31])
        gb_vf = eval(line[36:46])
        gb_ir = eval(line[48:])

        node_numbers.append(node_number)
        gb_irs.append(gb_ir)
        gb_vfs.append(gb_vf)

    return node_numbers, gb_vfs, gb_irs

    
def read_gb_file(filename_gb):
    """Read a first time the BG fiel to have a big picture

    Args:
        filename_gb ([type]): [description]

    Returns:
        [type]: [description]
    """
    with open(filename_gb, 'r') as f_gb:

        lines = f_gb.readlines()

    lines_with_new_node = []
    index_lines_with_new_node = []

    for i, line in enumerate(lines):
        
        if "Factors from node:" in line:
            lines_with_new_node.append(line)
            index_lines_with_new_node.append(i)

    list_nodes_numbers_bg = [ eval( l[19:24] ) for l in lines_with_new_node]
    
    return list_nodes_numbers_bg, index_lines_with_new_node, lines_with_new_node


def trans_dict_of_list_to_mat(dol, list_of_nodes_numbers):
    """transform the dist of list to a np.array

    Args:
        dol (dict of list): the dict of list generated earlier

    Returns:
        mat_gb: the np.array matrix
    """
    mat_gb = np.zeros( (len(list_of_nodes_numbers), len(list_of_nodes_numbers)) )

    for i, id in enumerate(list_of_nodes_numbers):
        for j, jd in enumerate(list_of_nodes_numbers):
            try:
                mat_gb[i, j]= dol[id][j]
            except KeyError:
                print(i, j, id, jd)


    return mat_gb


def get_all_dict_of_gb_lines(filename_gb, list_of_nodes_numbers, index_lines_with_new_node, lines_with_new_node):
    """open the file and read all the coefficients"""

    with open(filename_gb, 'r') as f_gb:

        lines = f_gb.readlines()


    dict_of_list_gb_vf = {}
    dict_of_list_gb_ir = {}

    for linenumber, linetitle in zip(index_lines_with_new_node, lines_with_new_node):

        node_number = eval(linetitle[19:24])
        line_start = linenumber+3

        for index, line in enumerate(lines[line_start:]):
            if "------------------" in line:
                line_stop = line_start+index
                break 



        lines_selected = lines[line_start:line_stop]

        nodes, gb_vfs, gb_irs = parse_one_node_lines(lines_selected)

        dict_gbvfs = { n: gb for n,gb in zip(nodes, gb_vfs )}
        dict_gbirs = { n: gb for n,gb in zip(nodes, gb_irs )}

        
        gbvf_every_nodes = []
        gbir_every_nodes = []
        for n in list_of_nodes_numbers:
            gbvf_every_nodes.append( dict_gbvfs.get(n, 0) )
            gbir_every_nodes.append( dict_gbirs.get(n, 0) )


        
        dict_of_list_gb_vf[node_number] = gbvf_every_nodes
        dict_of_list_gb_ir[node_number] = gbir_every_nodes

    #added empty nodes
    for n in list_of_nodes_numbers:
        if n not in dict_of_list_gb_vf.keys():
            dict_of_list_gb_vf[n] = [ 0 for n in list_of_nodes_numbers]

        if n not in dict_of_list_gb_ir.keys():
            dict_of_list_gb_ir[n] = [ 0 for n in list_of_nodes_numbers]
            
    mat_gb_vf = trans_dict_of_list_to_mat(dict_of_list_gb_vf, list_of_nodes_numbers)
    mat_gb_ir = trans_dict_of_list_to_mat(dict_of_list_gb_ir, list_of_nodes_numbers)

    return mat_gb_vf, mat_gb_ir


def get_nth_max_coupling(mat, list_of_nodes_names, n=10):
    """Read the matrice and print the biggest elements

    Args:
        mat (numpy.array): the coefficients matrix
        list_of_nodes_names (list): the list of names
        n (int, optional): number of maximum element to print. Defaults to 10.
    """
    ind = np.argsort(mat, None,)[-2*n:][::-1]
    ind_2d = np.unravel_index(ind, mat.shape)

    oposit_couples = []
    for ind_i, ind_j in zip(*ind_2d):
        if [ind_i, ind_j] not in oposit_couples:

            oposit_couples.append([ind_j, ind_i])

            print( f" {mat[ind_i, ind_j]:2.4f} between {list_of_nodes_names[ind_i]} and {list_of_nodes_names[ind_j]}")



# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-26 14:56:58
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-26 15:03:33

from pythermica.post_process import process_nwk_and_temperature
import pythermica

path = pythermica.__path__[0]+"/../exemples/model_test/"

process_nwk_and_temperature(path)



# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-24 15:49:41
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-25 09:39:52
import pytest

import pythermica
from pythermica.analyse_nwk import (extract_nodes, get_useful_files,
                                    generate_correlation_matrixes,
                                    read_gb_file,
                                    get_all_dict_of_gb_lines,
                                    analyse_GR_nwk_data,
                                    analyse_GL_nwk_data,
                                    get_nth_max_coupling)


def test_get_useful_files():
    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)

    assert filename_gb.name == "ionsat_deployed_4.WINTER.HC.1.gb.txt"
    assert filename_nod.name == "ionsat_deployed_4.WINTER.HC.1.nod.nwk"
    assert filename_gr.name == "ionsat_deployed_4.WINTER.HC.1.gr.nwk"
    assert filename_gl.name == "ionsat_deployed_4.WINTER.HC.1.gl.nwk"

  
def test_extract_nodes():
    
    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)

    list_of_nodes_numbers, list_of_nodes_names = extract_nodes(filename_nod)
    
    assert len(list_of_nodes_names) == len(list_of_nodes_numbers)
    
    
def test_analyse_GR_nwk_data():

    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)

    list_of_nodes_numbers, list_of_nodes_names = extract_nodes(filename_nod)

    mat_GR = analyse_GR_nwk_data(filename_gr,
                        list_of_nodes_numbers,
                        list_of_nodes_names)
    
    assert mat_GR.shape == (len(list_of_nodes_names), len(list_of_nodes_names))
    
    
def test_analyse_GL_nwk_data():

    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)

    list_of_nodes_numbers, list_of_nodes_names = extract_nodes(filename_nod)

    mat_GL = analyse_GL_nwk_data(filename_gl,
                                 list_of_nodes_numbers,
                                 list_of_nodes_names)

    assert mat_GL.shape == (len(list_of_nodes_names), len(list_of_nodes_names))


def test_generate_correlation_matrixes():

    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)

    list_of_nodes_numbers, list_of_nodes_names = extract_nodes(filename_nod)

    mat_GR = analyse_GR_nwk_data(filename_gr,
                                 list_of_nodes_numbers,
                                 list_of_nodes_names)

    fig, axarr = generate_correlation_matrixes([mat_GR],
                                            list_of_nodes_names,
                                            "mat_GR",
                                            )
    assert len(axarr) == 1


def test_read_gb_file():
    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)

    list_nodes_numbers_bg, index_lines_with_new_node, lines_with_new_node = read_gb_file(filename_gb)
    

def test_get_nth_max_coupling():
    
    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)
    
    list_of_nodes_numbers, list_of_nodes_names = extract_nodes(filename_nod)

    mat_GR = analyse_GR_nwk_data(filename_gr,
                                 list_of_nodes_numbers,
                                 list_of_nodes_names)
    
    get_nth_max_coupling(mat_GR, list_of_nodes_names, 5)


def test_get_all_dict_of_gb_lines():
    root = pythermica.__path__[0]+"/../exemples/model_test/"

    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        root)

    list_of_nodes_numbers, list_of_nodes_names = extract_nodes(filename_nod)

    list_nodes_numbers_bg, index_lines_with_new_node, lines_with_new_node = read_gb_file(
        filename_gb)

    mat_gb_vf, mat_gb_ir = get_all_dict_of_gb_lines(
        filename_gb,
        list_of_nodes_numbers,
        index_lines_with_new_node,
        lines_with_new_node)



# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-26 14:31:18
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-26 15:01:48

import pythermica
from pythermica import Thermica
from pythermica.analyse_nwk import *
from pythermica.plot_variables import *

def process_nwk_and_temperature( path_results,
                                output_dir=None,
                                nodes_names_to_discard=[]):
    
    """analyse everythin needed for one simulation"""
    
    
    if output_dir is None:
        output_dir = path_results
    
    """First the NWK data"""
    filename_gb, filename_nod, filename_gr, filename_gl = get_useful_files(
        path_results)

    list_of_nodes_numbers, list_of_nodes_names = extract_nodes(filename_nod)
    
    """Analysing the Gebhart Factor"""
    list_nodes_numbers_bg, index_lines_with_new_node, lines_with_new_node = read_gb_file(
        filename_gb,)

    """Is ther any missing factor ?"""
    
    print("missing Gebhart Factor entry for nodes : ")

    for idx, n in enumerate(list_of_nodes_numbers):
        if n not in list_nodes_numbers_bg:
            print(n, ":", list_of_nodes_names[idx])
    
    print("")
    """ generating the GB matrix figure"""
    
    mat_gb_vf, mat_gb_ir = get_all_dict_of_gb_lines(
        filename_gb, list_of_nodes_numbers, index_lines_with_new_node, lines_with_new_node)

    print("Generating Correlation Matrix for BG factors")
    # fig, axarr = generate_correlation_matrixes(list_mats=[mat_gb_ir, mat_gb_vf],
    #                                            list_node_names=list_of_nodes_names,
    #                                            list_titles=[
    #     "Gebhart factor IR", "Gebhart factor VF"],
    #     use_log=False,
    #     save_fig=True,
    #     filename=output_dir+"/matrices_Gebhart_factors_values",
    #     cmap="Blues")
    
    """ generating the GR matrix figure"""
    mat_GR = analyse_GR_nwk_data(
        filename_gr, list_of_nodes_numbers, list_of_nodes_names)
    
    print("Generating Correlation Matrix for GR coefs")

    fig, axarr = generate_correlation_matrixes(list_mats=[mat_GR],
                                               list_node_names=list_of_nodes_names,
                                               list_titles=[
                                                   "Radiative couplings (IR) computed by Thermica"],
                                               use_log=True,
                                               save_fig=True,
                                               filename=output_dir+"/matrices_IR_GR_factors_values"
                                               )
    
    mat_GL = analyse_GL_nwk_data(
        filename_gl, list_of_nodes_numbers, list_of_nodes_names)

    fig, axarr = generate_correlation_matrixes([mat_GL],
                                               list_of_nodes_names,
                                               ["conductives couplings computed by Thermica"],
                                               use_log=True,
                                               save_fig=True,
                                               filename=output_dir+"/matrices_GL_factors_values"
                                               )
    
    """Second, the thermal data"""
    
    thermal_result = Thermica(path_results)
    
    time_temperature = thermal_result.return_time_temperature()
    temperatures = thermal_result.get_temperature()

    fig, axarr = plt.subplots(4, 6, figsize=(8, 3))

    for ax, node_label, nodes_list_for_label in zip(axarr.flatten(),
                                                    thermal_result.names_unique,
                                                    thermal_result.nodes_per_names):
        if node_label == "Space Node":
            pass
        else:
            index_nodes_chassis = []
            for i, node in enumerate(thermal_result.nodes_value):
                if node in nodes_list_for_label:
                    index_nodes_chassis.append(i)

            index_nodes_chassis = np.array(index_nodes_chassis)

            ax.plot(time_temperature, temperatures.T[:, index_nodes_chassis])
            ax.set_title(node_label)
            ax.grid(c="grey")

    plt.savefig(output_dir +"/all_node_temp.png", dpi=300)
    
    figure_over_nodes(therm_results=[thermal_result],
                      temperatures=[temperatures],
                      times=[time_temperature],
                      n_orbits=8,
                      path_list=[path_results],
                      case_names=[""],
                      nodes_to_process=list_of_nodes_names,
                      path_root=output_dir,
                      name_yaxis="Temperature",
                      filename_prefix="")
    
    internal_dissipations = thermal_result.get_internal_dissipation()
    
    figure_over_nodes(therm_results=[thermal_result],
                      temperatures=[internal_dissipations],
                        times=[time_temperature],
                        n_orbits=8,
                        path_list=[path_results],
                        case_names=[""],
                        nodes_to_process=["/Powercard P60", "/Printed Circuits",
                                          "/Transponder (1)", "/Batteries",
                                          "/Propeller side front",
                                          "/Propeller side left",
                                          "/Propeller side right",
                                          "/Propeller side top",
                                          "/Propeller side back",
                                          "/Propeller side bottom", ],
                        path_root=output_dir,
                        name_yaxis="Internal Dissipation [W]",
                        filename_prefix="")
    
    totalInternalDissipation(thermal_result,
                             internal_dissipations,
                             time_temperature,
                             n_orbits = 8,
                             path_list = [path_results],
                             nodes_to_process=None,
                             path_root=output_dir,
                             filename_prefix="Total_IQ")



    





# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-21 13:21:38
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-25 09:33:24
import h5py


def descend_obj(obj,sep='\t'):
    """
    Iterate through groups in a HDF5 file and prints the groups and datasets names and datasets attributes
    """

    if type(obj) in [h5py._hl.group.Group, h5py._hl.files.File]:

        #print(obj.keys())
        for key in obj.keys():
            print(sep, '-', key, ':', obj[key])
            descend_obj(obj[key], sep=sep+'\t')

    elif type(obj)==h5py._hl.dataset.Dataset:
        keylist = ["unit", "time reference"]
        obj.attrs.keys()
        for key in keylist:
            try:
                print(sep+'\t', '-', key, ':', obj.attrs[key].decode(encoding="latin-1") )
            except KeyError:
                pass
            except UnicodeDecodeError:
                print(sep+'\t', '-', key, ':', obj.attrs[key] )



def h5dump(path,group='/'):
    """
    print HDF5 file metadata

    group: you can give a specific group, defaults to the root group
    """
    with h5py.File(path,'r') as f:

        descend_obj(f[group])



# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-21 13:21:38
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-25 09:41:05
#!/usr/bin/env python

"""Tests for `pythermica` package."""

import pytest

import pythermica
from pythermica import Thermica


def test_Thermica():
    """test Thermica Class with an example"""
    
    path = pythermica.__path__[0]+"/../exemples/simulation_1/results_1/"

    therm_results = Thermica(path, verbose=True)
    
    therm_results.dump_result_file_structure()
    
    sf_filenames = therm_results.get_filenames("solarflux")
    assert sf_filenames[0].name == "ionsat_deployed_3.0.sf.h5"
    
    pf_filenames = therm_results.get_filenames("earthflux")
    assert pf_filenames[0].name == "ionsat_deployed_3.0.pf.h5"

    temp_filenames = therm_results.get_filenames("temperature")
    assert temp_filenames[0].name == "ionsat_deployed.temp.h5"

    sf_filename = therm_results.get_solarflux_file()
    assert sf_filename.name == "ionsat_deployed_3.0.sf.h5"
    
    temp_filename = therm_results.get_temperature_file()
    assert temp_filename.name == "ionsat_deployed.temp.h5"
    
    value_sf, time_sf = therm_results.read_solarflux()
    
    assert len(time_sf) == 219
    assert len(value_sf) == 850
    
    value_temp = therm_results.get_temperature()
    assert value_temp.shape == (856, 1092)
    
    time_temp = therm_results.return_time_temperature()
    assert len(time_temp) == value_temp.shape[1]


# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-24 16:05:07
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-24 16:06:49

import pytest

import pythermica
from pythermica.hdf5_process import h5dump

def test_h5dump():
    
    h5dump(pythermica.__path__[0] + "/../exemples/simulation_1/results_1/ionsat_deployed.temp.h5")
    
    



# -*- coding: utf-8 -*-
# @Author: Antoine Tavant
# @Date:   2022-01-21 13:21:38
# @Last Modified by:   Antoine Tavant
# @Last Modified time: 2022-01-26 14:25:54
"""Top-level package for pythermica."""

__author__ = """Antoine Tavant"""
__email__ = 'antoine.tavant@lpp.polytechnique.fr'
__version__ = '0.1.2'

from .pythermica import Thermica
from .analyse_nwk import *
from .plot_variables import *
from .hdf5_process import *


